/*
 * Copyright (C) 2005, Wei Yuan.
 * All rights reserved.
 * 
 * Author: Wei Yuan
 * Created on: August 18, 2005
 * 
 */

using System;
using System.Data;
using System.Text;

namespace Cavingdeep.DbTestSample {
   /// <summary>
   /// Provides an employee's informations.
   /// </summary>
   [Serializable]
   public class EmployeeInfo {
      private string id;
      private string firstName;
      private string lastName;
      private DateTime birthday;

      /// <summary>
      /// Creates an instance.
      /// </summary>
      /// <param name="employeeId">Specifies the employee's ID.</param>
      /// <exception cref="ArgumentNullException">
      /// <c>employeeId</c> is null or empty.
      /// </exception>
      public EmployeeInfo(string employeeId) {
         if (employeeId == null || employeeId.Length == 0) {
            throw new ArgumentNullException("employeeId");
         }

         this.id = employeeId;
      }

      /// <summary>
      /// Gets an employee's informations.
      /// </summary>
      /// <param name="employeeId">Employee ID used to get related informations.</param>
      /// <param name="queryCommand">The query command used to get employee's data from database.</param>
      /// <returns>
      /// <see cref="EmployeeInfo"/> instance containing emloyee's informations.
      /// </returns>
      /// <remarks>
      /// You don't need to provide <c>queryCommand</c>'s command text, the correct query text
      /// is setted within this method.
      /// </remarks>
      /// <exception cref="NoConnectionException">
      /// Thrown when connection cannot be opened.
      /// </exception>
      /// <exception cref="Exception">
      /// Thrown when not able to read datas from the database.
      /// </exception>
      public static EmployeeInfo GetEmployeeInfo(string employeeId, IDbCommand queryCommand) {
         if (employeeId == null || employeeId.Length == 0) {
            throw new ArgumentNullException("employeeId");
         }
         if (queryCommand == null) {
            throw new ArgumentNullException("queryCommand");
         }

         queryCommand.CommandText = "SELECT id, firstName, lastName, birthday FROM Employees " +
                                    "WHERE id = '" + employeeId + "'";

         // Encapsulate exceptions during connection opening.
         try {
            queryCommand.Connection.Open();
         } catch (Exception ex) {
            throw new NoConnectionException(ex.Message, ex);
         }

         try {
            IDataReader reader = queryCommand.ExecuteReader();
            // If no record is found in database, return null.
            EmployeeInfo employeeInfo = null;

            if (reader.Read()) {
               employeeInfo = new EmployeeInfo(reader.GetString(0));

               employeeInfo.firstName = reader.GetString(1);
               employeeInfo.lastName = reader.GetString(2);
               employeeInfo.birthday = reader.GetDateTime(3);
            }

            return employeeInfo;
         } finally {
            queryCommand.Connection.Close();
         }
      }

      #region Properties.

      /// <summary>
      /// Gets this employee's ID.
      /// </summary>
      public string Id {
         get {return this.id;}
      }

      /// <summary>
      /// Gets/Sets this employee's first name.
      /// </summary>
      public string FirstName {
         get {return this.firstName;}
         set {this.firstName = value;}
      }

      /// <summary>
      /// Gets/Sets this employee's last name.
      /// </summary>
      public string LastName {
         get {return this.lastName;}
         set {this.lastName = value;}
      }

      /// <summary>
      /// Gets this employee's full name.
      /// </summary>
      public string FullName {
         get {return this.lastName + ' ' + this.firstName;}
      }

      /// <summary>
      /// Gets/Sets this employee's birthday.
      /// </summary>
      public DateTime Birthday {
         get {return this.birthday;}
         set {this.birthday = value;}
      }

      #endregion

      #region Object Members
      
      /// <summary>
      /// Returns a string representation of this instance.
      /// </summary>
      public override string ToString() {
         StringBuilder builder = new StringBuilder();
         builder.AppendFormat("[Employee {0}", this.FullName);
         builder.Append(Environment.NewLine);
         builder.AppendFormat("ID: {0}", this.id);
         builder.Append(Environment.NewLine);
         builder.AppendFormat("Birthday: {0}", this.birthday.ToShortDateString());
         builder.Append(']');

         return builder.ToString();
      }
      
      /// <summary>
      /// Gets the hash code for this instance.
      /// </summary>
      public override int GetHashCode() {
         return this.ToString().GetHashCode();
      }
      
      /// <summary>
      /// Determines whether the specified instance is equal to this instance.
      /// </summary>
      public override bool Equals(object obj) {
         EmployeeInfo objToCompare = obj as EmployeeInfo;
         
         if (objToCompare == null) {
            return false;
         }
         
         return (this.GetHashCode() == objToCompare.GetHashCode());
      }
      
      #endregion
   }
}
